/*
 * CPble_hci.c
 *
 * Created: 6/30/2017 1:19:10 PM
 *  
 */ 
//#include <AltSoftSerial.h>
#include "CPble_typedef.h"
#include "CPble_hci.h"
#include <string.h>
#include <stdio.h>
#include <stdint.h>


uint8_t GAP_DeviceInit( uint8_t taskID, uint8_t profileRole, uint8_t maxScanResponses, uint8_t *pIRK, uint8_t *pSRK, uint32_t *pSignCounter )
{
	uint8_t buf[42];
	uint8_t len = 0;
	
	buf[len++] = 0x01;                  // -Type    : 0x01 (Command)
	buf[len++] = 0x00;                  // -Opcode  : 0xFE00 (GAP_DeviceInit)
	buf[len++] = 0xFE;
	
	buf[len++] = 0x26;                  // -Data Length 0x26bytes (38bytes) = 1 + 1 + 16 + 16 + 4
	buf[len++] = profileRole;           //  Profile Role
	buf[len++] = maxScanResponses;      //  MaxScanRsps
	memcpy(&buf[len], pIRK, 16);        //  IRK // copy 16bytes from pIRK to buf[]
	len += 16;
	memcpy(&buf[len], pSRK, 16);        //  SRK // copy 16bytes from pSRK to buf[]
	len += 16;
	memcpy(&buf[len], pSignCounter, 4); //  SignCounter // copy 4bytes from pSignCounter to buf[] 
	len += 4;
	
	UART0_Init();
	SerialPrintln0(buf, len);
	//AltSoftSerial.write(buf, len);
	
	return 1;
}

uint8_t GAP_DeviceDiscoveryRequest( gapDevDiscReq_t *pParams )
{
	uint8_t buf[20];
	uint8_t len = 0;
	
	buf[len++] = 0x01;                  // -Type    : 0x01 (Command)
	buf[len++] = 0x04;                  // -Opcode  : 0xFE04 (GAP_DeviceDiscoveryRequest)
	buf[len++] = 0xFE;
	
	buf[len++] = 0x03;                 // -Data Length 0x03bytes (3bytes) = 1 + 1 + 1
	buf[len++] = pParams->mode;        //  Mode
	buf[len++] = pParams->activeScan;  //  ActiveScan
	buf[len++] = pParams->whiteList;   //  WhiteList
	
	//  hci_tl_write(buf, len);
	//  hci_tl_wait_for_response();
	UART0_Init();
	SerialPrintln0(buf, len);
	//AltSoftSerial.write(buf, len);
	

	return 1;
}

uint8_t GAP_EstablishLinkReq( gapEstLinkReq_t *pParams )
{
	uint8_t buf[20];
	uint8_t len = 0;
	
	buf[len++] = 0x01;   // 
	memcpy(&buf[len], "\x09\xFE", 2);
	len += 2;
	
	buf[len++] = 0x03 + B_ADDR_LEN;
	
	buf[len++] = pParams->highDutyCycle;
	buf[len++] = pParams->whiteList;
	buf[len++] = pParams->addrTypePeer;
	memcpy(&buf[len], pParams->peerAddr, B_ADDR_LEN);
	len+=B_ADDR_LEN;

	UART0_Init();
	SerialPrintln0(buf, len);
	return 1;
}

uint8_t GATT_WriteCharValue( uint16_t connHandle, attWriteReq_t *pReq, uint8_t taskId )
{
	uint8_t buf[20];
	uint8_t len = 0;
	
	buf[len++] = 0x01;
	memcpy(&buf[len], "\x92\xFD", 2);
	len += 2;
	
	buf[len++] = 0x04 + pReq->len;
	
	memcpy(&buf[len], &connHandle, 2);
	len += 2;
	memcpy(&buf[len], &pReq->handle, 2);
	len += 2;
	memcpy(&buf[len], pReq->value, pReq->len);
	len += pReq->len;
	
	UART0_Init();
	SerialPrintln0(buf, len);

	return 1;
}

uint8_t GATT_WriteNoRsp( uint16_t connHandle, attWriteReq_t *pReq, uint8_t taskId )
{
	uint8_t buf[20];
	uint8_t len = 0;
	
	buf[len++] = 0x01;
	memcpy(&buf[len], "\xB6\xFD", 2);
	len += 2;
	
	buf[len++] = 0x04 + pReq->len;
	
	memcpy(&buf[len], &connHandle, 2);
	len += 2;
	memcpy(&buf[len], &pReq->handle, 2);
	len += 2;
	memcpy(&buf[len], pReq->value, pReq->len);
	len += pReq->len;
	UART0_Init();
	SerialPrintln0(buf, len);
	//AltSoftSerial.write(buf, len);
	return 1;
}

/***********************************
* Master device
*************************************/
/*
void biscuit_central_init()
{
	GAPCentralRole_StartDevice();
}
static uint8_t gapCentralRoleTaskId = 0;
static uint8_t  gapCentralRoleIRK[KEYLEN] = {0};
static uint8_t  gapCentralRoleSRK[KEYLEN] = {0};
static uint32_t gapCentralRoleSignCounter = 1;
static uint8_t  gapCentralRoleBdAddr[B_ADDR_LEN];
static uint8_t  gapCentralRoleMaxScanRes = 5;
*/

static uint8_t gapCentralRoleTaskId = 0;
static uint8_t  gapCentralRoleIRK[KEYLEN] = {0};
static uint8_t  gapCentralRoleSRK[KEYLEN] = {0};
static uint32_t gapCentralRoleSignCounter = 1;
static uint8_t  gapCentralRoleBdAddr[B_ADDR_LEN];
static uint8_t  gapCentralRoleMaxScanRes = 5;

uint8_t GAPCentralRole_StartDevice()
{
	// gapCentralRoleTaskId = 0, 
	// GAP_PROFILE_CENTRAL = 0x08
	// gapCentralRoleMaxScanRes = 5
	// gapCentralRoleIRK = gapCentralRoleIRK[KEYLEN] = {0}; // KEYLEN= 16
	// gapCentralRoleSRK = {0};
	// gapCentralRoleSignCounter = 1
	return GAP_DeviceInit( gapCentralRoleTaskId, GAP_PROFILE_CENTRAL,
	gapCentralRoleMaxScanRes, gapCentralRoleIRK,
	gapCentralRoleSRK, &gapCentralRoleSignCounter );
}

uint8_t GAPCentralRole_EstablishLink( uint8_t highDutyCycle, uint8_t whiteList, uint8_t addrTypePeer, uint8_t *peerAddr )
{
	gapEstLinkReq_t params;

	params.taskID = gapCentralRoleTaskId;
	params.highDutyCycle = highDutyCycle;
	params.whiteList = whiteList;
	params.addrTypePeer = addrTypePeer;
	memcpy( params.peerAddr, peerAddr, B_ADDR_LEN );

	return GAP_EstablishLinkReq( &params );
}

uint8_t GAPCentralRole_StartDiscovery( uint8_t mode, uint8_t activeScan, uint8_t whiteList )
{
	gapDevDiscReq_t params;

	params.taskID = gapCentralRoleTaskId;
	params.mode = mode;
	params.activeScan = activeScan;
	params.whiteList = whiteList;

	return GAP_DeviceDiscoveryRequest( &params );
}

